use criterion::{criterion_group, criterion_main, Criterion};
use iteration::iter::*;

const N_SUM: usize = 10000000;
const N_FACTOR: usize = 10000;
const M: u64 = 50;

pub fn criterion_benchmark(c: &mut Criterion) {
    let mut group = c.benchmark_group("Sum Values");
    let v = rand_array(N_SUM, M);

    group.bench_function("sum_with_index", |b| b.iter(|| sum_with_index(&v)));
    group.bench_function("sum_with_loop", |b| b.iter(|| sum_with_loop(&v)));
    group.bench_function("sum_with_iter", |b| b.iter(|| sum_with_iter(&v)));
    group.bench_function("sum_with_par_iter", |b| b.iter(|| sum_with_par_iter(&v)));
    group.finish();

    let mut group = c.benchmark_group("Count Factors");
    let v = rand_array(N_FACTOR, M);

    group.bench_function("factors_with_index", |b| b.iter(|| factors_with_index(&v)));
    group.bench_function("factors_with_loop", |b| b.iter(|| factors_with_loop(&v)));
    group.bench_function("factors_with_iter", |b| b.iter(|| factors_with_iter(&v)));
    group.bench_function("factors_with_par_iter", |b| {
        b.iter(|| factors_with_par_iter(&v))
    });
    group.finish();
}

criterion_group!(benches, criterion_benchmark);
criterion_main!(benches);
